/***
 * Julia
 * Copyright (C) 2005-2006 INRIA, France Telecom, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliak.control.attribute;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.AttributeController;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.julia.BasicComponentInterface;
import org.objectweb.fractal.julia.Controller;
import org.objectweb.fractal.julia.InitializationContext;
import org.objectweb.fractal.julia.control.attribute.CloneableAttributeController;
import org.objectweb.fractal.juliak.FractalHelper;
import org.objectweb.fractal.juliak.asm.ClassGenerator;
import org.objectweb.fractal.juliak.asm.CloneableAttributeControllerClassGenerator;
import org.objectweb.fractal.juliak.factory.InstanceGenerator;


/**
 * Implementation of the attribute controller.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 * @since 2.5
 */
public class AttributeControllerImpl
    implements CloneableAttributeController, Controller {
    
    public AttributeControllerImpl() {}

    
    // --------------------------------------------------------------
    // CloneableAttributeController implementation
    // --------------------------------------------------------------
    
    public void cloneFcAttributes(AttributeController dst) {
        delegate.cloneFcAttributes(dst);
    }
    
    private CloneableAttributeController delegate;

    
    // --------------------------------------------------------------
    // Controller implementation
    // --------------------------------------------------------------
    
    public void initFcController(InitializationContext ic)
    throws InstantiationException {        
        
        Component comp = (Component) ic.interfaces.get("component");
        
        /*
         * Retrieve the attribute-controller interface.
         */
        Object itf = FractalHelper.getAttributeController(comp);
        if( ! (itf instanceof BasicComponentInterface) ) {
            final String msg =
                "The interface for attribute-controller should implement o.o.f.j.BasicComponentInterface";
            throw new InstantiationException(msg);
        }
        BasicComponentInterface citf = (BasicComponentInterface) itf;
        InterfaceType it = (InterfaceType) citf.getFcItfType();
        
        /*
         * Generate a cloneable (in the sense CloneableAttributeController)
         * version of the attribute-controller.
         */
        if( ig == null ) {
            ClassGenerator caccg =
                new CloneableAttributeControllerClassGenerator();
            ig = new InstanceGenerator(caccg);
        }
        Object o = ig.generate(it, null);
        if( !(o instanceof CloneableAttributeController) ) {
            final String msg =
                "The generated instance should implement o.o.f.j.control.attribute.CloneableAttributeController";
            throw new InstantiationException(msg);
        }
        delegate = (CloneableAttributeController) o;
        
        /*
         * Replace the original implementation of the attribute-controller by
         * the generated version.
         */
        citf.setFcItfImpl(o);
    }
    
    private InstanceGenerator ig;
}
