/***
 * Julia
 * Copyright (C) 2005-2006 INRIA, France Telecom, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliak.membrane;


/**
 * This class defines the basic membranes which are defined by Julia.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 * @since 2.5
 */
public class BasicMembranes {

    /**
     * Return the definitions of membranes supported by Juliak.
     */
    public MembraneDef[] getMembraneDefs() {
        return MEMBRANES;
    }

    
    // ------------------------------------------------------
    // Definition of the default membrane supported by Juliak 
    // ------------------------------------------------------
    
    final private static MembraneDef[] MEMBRANES =
        new MembraneDef[]{
        
            // ---------
            // Bootstrap
            // ---------
            new MembraneDef(
                    "bootstrap",
                    "org.objectweb.fractal.juliak.membrane.Bootstrap"),
            
            // -------
            // Regular
            // -------
            new MembraneDef(
                    "primitive",
                    "org.objectweb.fractal.juliak.membrane.Primitive"),
            new MembraneDef(
                    "composite",
                    "org.objectweb.fractal.juliak.membrane.Composite"),
    
            // ----------
            // Parametric
            // ----------
            new MembraneDef(
                    "parametricPrimitive",
                    "org.objectweb.fractal.juliak.membrane.ParametricPrimitive"),
            new MembraneDef(
                    "parametricComposite",
                    "org.objectweb.fractal.juliak.membrane.ParametricComposite"),
    
            // --------
            // Template
            // --------
            new MembraneDef(
                    "primitiveTemplate",
                    "org.objectweb.fractal.juliak.membrane.PrimitiveTemplate"),
            new MembraneDef(
                    "compositeTemplate",
                    "org.objectweb.fractal.juliak.membrane.CompositeTemplate"),
    
            // -------------------
            // Parametric template
            // -------------------
            new MembraneDef(
                    "parametricPrimitiveTemplate",
                    "org.objectweb.fractal.juliak.membrane.ParametricPrimitiveTemplate"),
            new MembraneDef(
                    "parametricCompositeTemplate",
                    "org.objectweb.fractal.juliak.membrane.ParametricCompositeTemplate"),
            
            // ------------
            // Auto-binding
            // ------------
            new MembraneDef(
                    "autoBindingPrimitive",
                    "org.objectweb.fractal.juliak.membrane.AutoBindingPrimitive"),
            new MembraneDef(
                    "autoBindingComposite",
                    "org.objectweb.fractal.juliak.membrane.AutoBindingComposite"),

            // --------------------
            // Flat component model
            // --------------------
            new MembraneDef(
                    "flatPrimitive",
                    "org.objectweb.fractal.juliak.membrane.FlatPrimitive"),
            new MembraneDef(
                    "flatParametricPrimitive",
                    "org.objectweb.fractal.juliak.membrane.FlatParametricPrimitive"),
            new MembraneDef(
                    "flatPrimitiveTemplate",
                    "org.objectweb.fractal.juliak.membrane.FlatPrimitiveTemplate"),
            new MembraneDef(
                    "flatParametricPrimitiveTemplate",
                    "org.objectweb.fractal.juliak.membrane.FlatParametricPrimitiveTemplate"),

            // -------------------------------------------
            // Membrane definitions for control components
            // -------------------------------------------
            new MembraneDef("mPrimitive",null),
            new MembraneDef("mComposite",null)
        };
    
}
