/***
 * Julia
 * Copyright (C) 2005-2006 INRIA, France Telecom, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliak.membrane;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;


/**
 * Membrane repository. This class is a singleton.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 * @since 2.5
 */
public class Membranes {

    /** The singleton instance of itself. */
    private static Membranes singleton = new Membranes();
    private Membranes() {
        // Register Juliak membranes
        register(new JuliakMembranes().getMembraneDefs());
    }
    
    /**
     * Return the singleton instance of this class.
     */
    public static Membranes get() { return singleton; }
    
    
    // <controller desc, membrane def>
    /** Membrane definitions. */
    private Map mdefs = new HashMap();
    
    
    /**
     * Register a new membrane.
     * 
     * @param mdef  the membrane definition to register
     */
    public void register( MembraneDef mdef ) {
        
        Object controllerdesc = mdef.getControllerDesc();
        if( mdefs.containsKey(controllerdesc) ) {
            throw new IllegalArgumentException(
                    "Membrane "+controllerdesc+" is already registered" );
        }
        mdefs.put(controllerdesc,mdef);
    }
    
    /**
     * Register new membranes.
     */
    public void register( MembraneDef[] mdefs ) {
       for (int i = 0; i < mdefs.length; i++) {
           register(mdefs[i]);
       }
    }
    
    /**
     * Return true if the given controller description is registered with the
     * repository.
     */
    public boolean containsControllerDesc( Object controllerDesc ) {
        return mdefs.containsKey(controllerDesc);
    }
    
    /**
     * Return the {@link MembraneDef} instance associated to the given
     * controller description.
     */
    public MembraneDef getMembraneDef( Object controllerDesc ) {
        return (MembraneDef) mdefs.get(controllerDesc);
    }
    
    /**
     * Return the {@link MembraneDef} instances currently registered.
     */
    public Collection getMembraneDefs() {
        return mdefs.values();
    }
}
